@@ -45,6 +45,7 @@ function( object,
           ... ){
 
   # extract original X and Z
+  check_tinyVAST_version( object$internal$packageVersion )
   if(missing(newdata)){
     newdata = object$data
   }
@@ -237,6 +238,7 @@ function( object,
   ){
 
   # extract original X and Z
+  check_tinyVAST_version( object$internal$packageVersion )
   if(missing(newdata)) newdata = object$data
   # Build new .. object$data must be same as used for fitting to get SEs / skewness of random effects
   tmb_data2 = add_predictions( object = object,
@@ -472,6 +474,20 @@ function( object,
     # stream network
     A_gs = sfnetwork_evaluator( stream = object$spatial_domain$stream,
                                 loc = as.matrix(newdata[,object$internal$space_columns]) )
+  }else if( is(object$spatial_domain, "sfc_GEOMETRY") ){
+    sf_coords = st_as_sf( newdata,
+                          coords = object$internal$space_columns,
+                          crs = st_crs(object$spatial_domain) )
+    s_i = as.integer(st_within( sf_coords, object$spatial_domain ))
+    if(any(is.na(s_i))){
+      stop("Some rows of `newdata` in `predict` are not within the SAR domain.
+            Please exclude rows listed below or refit model with extended domain.\n",
+            paste0(which(is.na(s_i)), collapse = ", ") )
+    }
+    A_gs = sparseMatrix( i = seq_along(s_i),
+                         j = s_i,
+                         x = 1,
+                         dims = c(length(s_i),length(object$spatial_domain)) )
   }else{
     A_gs = matrix(1, nrow=nrow(newdata), ncol=1)    # dgCMatrix
     A_gs = as(Matrix(A_gs),"CsparseMatrix")
@@ -481,10 +497,18 @@ function( object,
   # Turn of t_i and c_i when times and variables are missing, so that delta_k isn't built
   if( length(object$internal$times) > 0 ){
     t_g = match( newdata[,object$internal$time_column], object$internal$times )
-  }else{ t_g = integer(0) }
+  }else{
+    t_g = integer(0)
+  }
   if( length(object$internal$variables) > 0 ){
+    # Telescoping for a univariate model using var = "response" by default
+    if( !(object$internal$variable_column %in% colnames(newdata)) ){
+      newdata = data.frame( newdata, matrix(object$internal$variables[1], nrow=nrow(newdata), ncol=1, dimnames=list(NULL,object$internal$variable_column)) )
+    }
     c_g = match( newdata[,object$internal$variable_column], object$internal$variables )
-  }else{ c_g = integer(0) }
+  }else{
+    c_g = integer(0)
+  }
 
   #
   AepsilonG_zz = cbind(predAtriplet$i, predAtriplet$j, t_g[predAtriplet$i], c_g[predAtriplet$i])
@@ -520,6 +544,10 @@ function( object,
     newdata = cbind( newdata, matrix(1, nrow=nrow(newdata), ncol=1, dimnames=list(NULL,object$internal$distribution_column)) )
   }
 
+  #
+  if( !(object$internal$distribution_column %in% colnames(newdata)) ){
+    newdata = cbind( newdata, matrix(1, nrow=nrow(newdata), ncol=1, dimnames=list(NULL,object$internal$distribution_column)) )
+  }
   # Error checks
   tmb_data2 = object$tmb_inputs$tmb_data
   if( ncol(tmb_data2$X_ij) != ncol(covariates$X_gj) ) stop("Check X_gj")
